// ChromGraphView.cpp : implementation of the CChromGraphView class
//

#include "stdafx.h"
#include "ChromGraph.h"

#include "ChromGraphDoc.h"
#include "ChromGraphView.h"
#include "GraphicsComponents.h"

#include <gdiplus.h>
using namespace Gdiplus;


#ifdef _DEBUG
#define new DEBUG_NEW
#endif



// CChromGraphView

IMPLEMENT_DYNCREATE(CChromGraphView, CView)

BEGIN_MESSAGE_MAP(CChromGraphView, CView)
	// Standard printing commands
	ON_COMMAND(ID_FILE_PRINT, CView::OnFilePrint)
	ON_COMMAND(ID_FILE_PRINT_DIRECT, CView::OnFilePrint)
	ON_COMMAND(ID_FILE_PRINT_PREVIEW, CView::OnFilePrintPreview)
END_MESSAGE_MAP()

// CChromGraphView construction/destruction

CChromGraphView::CChromGraphView()
{
	// TODO: add construction code here

}

CChromGraphView::~CChromGraphView()
{
}

BOOL CChromGraphView::PreCreateWindow(CREATESTRUCT& cs)
{
	// TODO: Modify the Window class or styles here by modifying
	//  the CREATESTRUCT cs

	return CView::PreCreateWindow(cs);
}

// CChromGraphView drawing

void transformBits(Bitmap* map);
void vectorGraphics(Graphics& where, int squareSize);
		
void CChromGraphView::OnDraw(CDC* pDC)
{
	CChromGraphDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);
	if (!pDoc)
		return;

	pDC->DrawText(CString("Hello World"), CRect(40,40,200,200), 0);   

// GDIPlus
	{
		const int squareSize = 101;

		Bitmap bmp(squareSize, squareSize, PixelFormat32bppARGB);
		Graphics out(*pDC);
		Graphics graphics(&bmp);
		Pen pen(Color(255, 255, 226, 64));  // opaque green
		
		SolidBrush grey(Color(255, 128, 128, 128));
		
		graphics.SetSmoothingMode( SmoothingModeNone);
		graphics.FillRectangle(&grey, 0, 0, squareSize, squareSize);
		
		vectorGraphics(graphics, squareSize);

		int i;
		for (i = 0; i < 64 ; i++){
			int j;
			for(j = 0; j < 2; j++){
				transformBits(&bmp);					// average
				vectorGraphics(graphics, squareSize);   // reinforce
			}
			out.DrawImage(&bmp, 10 + (i/8)*(squareSize+10), 100 + (i%8)*(squareSize+10));
		}
   }
}


void vectorGraphics(Graphics& graphics, int squareSize){
	Pen pen(Color(255, 255, 226, 64)); 
	graphics.DrawLine(&pen, squareSize/2, 0, squareSize/2, squareSize);
	graphics.DrawLine(&pen, 0, squareSize/2, squareSize, squareSize/2);

	Pen  pink(Color(255, 255, 200, 200));
	Pen   red(Color(255, 255,   0,   0));
	Pen green(Color(255,   0, 255,   0));
	Pen  blue(Color(255,   0,   0, 255));
	Pen trans(Color(0,   0, 0, 0));

	graphics.DrawLine(&trans,   0,0,0, squareSize-1);  //top
	graphics.DrawLine(&red  , 0,0,squareSize-1,0); //left
	graphics.DrawLine(&green,  0,squareSize-1, squareSize, squareSize-1); //bottom
	graphics.DrawLine(&blue,  squareSize-1,0, squareSize-1, squareSize-1);// right

	
	GraphicsContainer gc = graphics.BeginContainer();
	graphics.RotateTransform(360 - 30);
	graphics.ScaleTransform(float(.6), float(.6));
	graphics.DrawEllipse(&pink, squareSize/4,squareSize/4, squareSize/3, squareSize);
	graphics.ScaleTransform(float(.6), float(1.));
	graphics.RotateTransform(60);
	graphics.DrawEllipse(&pink, squareSize,squareSize/4, squareSize/3, squareSize);
	graphics.EndContainer(gc);

}


template<class _T>
class Janitor {
	_T *_p;
public:
	Janitor(_T* p ): _p(p){};
	~Janitor(){ delete _p; };
	_T* get(){return _p;};
};

// The transform is a nearest neighbor averaging.  There appears to be a darkening possibly caused by
// round down error.  I clone  the buffer of the entire plane so as to simplify what needs to be preserved
// for the averaging.  I first did this computation with just byte pointers and int accumulators, but it got
// pretty ugly quickly.  Created a PixelCell class instead adding a little bit of overhead, but its at least 
// more understandable
void transformBits(Bitmap* map){
	
	int wi  = map->GetWidth() - 1; // width in pixels (disregard last col)
	int hi  = map->GetHeight() - 1; // height in pixels (disregard last row)

	BitmapLocker dest(map);
	Janitor<Bitmap> mapCpy(map->Clone(0,0, map->GetWidth(), map->GetHeight(), map->GetPixelFormat()));
	BitmapLocker src(mapCpy.get());

	int  row;
	for (row = 1; row < hi; row++ ){
		int col;  
		PixelCell p(src.getPixel(row-1, 1)), ql(src.getPixel(row, 0)), 
			q(src.getPixel(row, 1)), qr(src.getPixel(row, 2)), r(src.getPixel(row+1, 1)), qq(dest.getPixel(row, 1));
		for(col = 1; col < wi; col++, ++p, ++ql, ++q, ++qr, ++r, ++qq){
	// cell iterator - 4 way			
	//              init		 eol		final
	//     p 		top   lhs+1			   	bot-2 rhs-1
	//  ql q qr		top+1 lhs+1  rhs		bot-1 rhs-1 
	//     r 		top+2 lhs+1             bot   rhs-1
	// q iterates does not visit boarder pixels so right, left, p and r are always valid
	// qq results
			qq = (p + q + r + ql + qr)/5;
		}
	}		
}


// CChromGraphView printing

BOOL CChromGraphView::OnPreparePrinting(CPrintInfo* pInfo)
{
	// default preparation
	return DoPreparePrinting(pInfo);
}

void CChromGraphView::OnBeginPrinting(CDC* /*pDC*/, CPrintInfo* /*pInfo*/)
{
	// TODO: add extra initialization before printing
}

void CChromGraphView::OnEndPrinting(CDC* /*pDC*/, CPrintInfo* /*pInfo*/)
{
	// TODO: add cleanup after printing
}


// CChromGraphView diagnostics

#ifdef _DEBUG
void CChromGraphView::AssertValid() const
{
	CView::AssertValid();
}

void CChromGraphView::Dump(CDumpContext& dc) const
{
	CView::Dump(dc);
}

CChromGraphDoc* CChromGraphView::GetDocument() const // non-debug version is inline
{
	ASSERT(m_pDocument->IsKindOf(RUNTIME_CLASS(CChromGraphDoc)));
	return (CChromGraphDoc*)m_pDocument;
}
#endif //_DEBUG


// CChromGraphView message handlers
