// GraphicsComponents.h
// Chris D'Urso
// 28 August 2005


#include <gdiplus.h>
using namespace Gdiplus;


class BitmapLocker;

// class PixelCell permits 32 bit math of each pixel channel
// initializing from pix map 8 byte a-r-g-b on object creation and on ++ operator
// setting pix map a-r-g-b only on assignment operator
class PixelCell{
	static const int pixel_size;
	unsigned int argb[ 4 /* pixel_size */];
public:
		
	PixelCell& operator++();
	PixelCell& operator+(const PixelCell& rhs);
	PixelCell& operator/(int rhs);
	PixelCell& operator=(const PixelCell& rhs);
	
	PixelCell(const PixelCell& __p){_p = __p._p; update(); }

private:
	friend BitmapLocker;
	PixelCell(void* base){ _p = static_cast<unsigned char*>(base); update();};
	unsigned char* _p;

	void update();
};



// locks exposes and releases Bitmap
class BitmapLocker{
public:
	BitmapLocker(Bitmap* bm){
		
		_bm = bm;
	
		Status out  = bm->LockBits( &Rect(0,0, bm->GetWidth(), bm->GetHeight()),
		ImageLockModeWrite, bm->GetPixelFormat(), &bmd);
	

		if(out)
			throw("LockBits error");
	}

	~BitmapLocker(){
		_bm->UnlockBits(&bmd);
	}

	int stride()const { return bmd.Stride; };
	void* scan(){ return bmd.Scan0; };

	PixelCell getPixel(int row, int col){  return PixelCell(static_cast<char*>(scan())+ stride()*row + col*PixelCell::pixel_size); };
	
private:
	BitmapData bmd;
	Bitmap* _bm;
};
